//  RobotArmGUI.pde
//
//  Ricardo Caja Calleja, 2017
//
//  Website: http://funwithcables.wordpress.com

ControlP5 cp5;
import processing.serial.*;  // Import serial library
import controlP5.*;  // Import controlP5 library
Serial myPort;  // Create object from Serial class
//Declare joint angles with initial angle (90 deg.) as integers:
int Base = 90;
int Shoulder = 90;
int Elbow = 90;
int Wrist1 = 90;
int Wrist2 = 90;
int End_effector = 45;
boolean Inv_Kin = false;  // Declare Inv_Kin boolean (this will allow us to change between modes 2 and 3)
// Declare X,Y,Z coordinates in string format:
String X_coord = "0";
String Y_coord = "0";
String Z_coord = "0";

void setup() {

  String portName = Serial.list()[4];  // Declare name for serial port (change the "4" accordingly to match your port)
  myPort = new Serial(this, portName, 9600);  // Declare serial port
  printArray(Serial.list());  // This shows all the serial ports in your computer
  myPort.write("1");  // Initialization function: write something to serial port (e.g., "1")
  delay(20);
  size(700,400);  // Define size (in px) of GUI
  noStroke();
  cp5 = new ControlP5(this);  // Needed for creating controlP5 objects
  PFont font = createFont("arial",20);  // Define font type and size
  cp5.addToggle("Inv_Kin",false,290,20,20,20);  // Define toggle button, inactive by default

  cp5.addSlider("Base")  // Define slider for Base
     .setPosition(100,240)
     .setSize(40,100)
     .setRange(0,180)
     ;
  cp5.addSlider("Shoulder")  // Define slider for Shoulder
     .setPosition(200,240)
     .setSize(40,100)
     .setRange(0,180)
     ;
  cp5.addSlider("Elbow")  // Define slider for Elbow
     .setPosition(300,240)
     .setSize(40,100)
     .setRange(0,180)
     ;
  cp5.addSlider("Wrist1")  // Define slider for Wrist1
     .setPosition(400,240)
     .setSize(40,100)
     .setRange(0,180)
     ;
  cp5.addSlider("Wrist2")  // Define slider for Wrist2
     .setPosition(500,240)
     .setSize(40,100)
     .setRange(0,180)
     ;
  cp5.addSlider("End_effector")  // Define slider for End_effector
     .setPosition(600,100)
     .setSize(40,200)
     .setRange(0,45)
     ;
  cp5.addTextfield("X_coord")  // Define text box for the X coordinate
     .setPosition(200, 80)
     .setSize(40, 40)
     .setFont(font)
     .setAutoClear(false)
     ;
  cp5.addTextfield("Y_coord")  // Define text box for the Y coordinate
     .setPosition(280, 80)
     .setSize(40, 40)
     .setFont(font)
     .setAutoClear(false)
     ;              
  cp5.addTextfield("Z_coord")  // Define text box for the Z coordinate
     .setPosition(360, 80)
     .setSize(40, 40)
     .setFont(font)
     .setAutoClear(false)
     ;      
  cp5.addBang("Submit")  // Define Submit button
     .setPosition(260, 170)
     .setSize(80, 20)
     .setTriggerEvent(Bang.RELEASE)
     ;   
}

void draw() {
  
  background(100,100,100);  // Define color of GUI background
  
  if(Inv_Kin==false){  // Proceed if the Inv_Kin toggle button is inactive 
  
    // Convert numerical angle values into strings with 3 digits:
    String Base_s = nf(Base, 3);           
    String Shoulder_s = nf(Shoulder, 3);
    String Elbow_s = nf(Elbow, 3);
    String Wrist1_s = nf(Wrist1, 3);
    String Wrist2_s = nf(Wrist2, 3);
    String End_effector_s = nf(End_effector, 3);
  
    // Write angle values (string format) to serial port:
    myPort.write(Base_s);
    delay(10);  
    myPort.write(Shoulder_s);
    delay(10);
    myPort.write(Elbow_s);
    delay(10);
    myPort.write(Wrist1_s);
    delay(10);
    myPort.write(Wrist2_s);
    delay(10);
    myPort.write(End_effector_s);
    delay(10);
    myPort.write("\r\n");  // Write empty line to serial port  
    delay(200);
    
    println(Base_s, Shoulder_s, Elbow_s, Wrist1_s, Wrist2_s, End_effector_s);  // The above strings can be printed in Processing terminal for debugging
  }
}

void Submit() {
  // Obtain X,Y,Z coordinates from GUI in string format:
  X_coord = cp5.get(Textfield.class,"X_coord").getText();
  Y_coord = cp5.get(Textfield.class,"Y_coord").getText();
  Z_coord = cp5.get(Textfield.class,"Z_coord").getText();

  // Convert X,Y,Z coordinates into integer format (to be able to make calculations with them):
  int X = int(X_coord);
  int Y = int(Y_coord);
  int Z = int(Z_coord);

// CALCULATION OF INVERSE KINEMATICS:

  float phi = atan(Y/X);
  float X1 = X/cos(phi);
  float Xw = X1 - 150;
  int a = 99;
  int b = 75;
  float c = sqrt(Xw*Xw+((Z-57)*(Z-57)));
  float C = acos((a*a+b*b-c*c)/(2*a*b));
  float A = asin((a/c)*sin(C));
  float B = asin((b/c)*sin(C));
  float tau = acos(Xw/c);
  float n = asin(Xw/c);
  float m = B + n + PI/2;  

// Calculate servo angles (in string format):
// 90 deg. (PI/2 rad.) had to be added to Base, Shoulder and Wrist1.
// 30 deg. (PI/6 rad.) had to be added to Elbow.
// This is because the servos input range is 0 to 180 deg. (not -90 to 90 deg., or -30 to 150 deg.)
  String Base_s = nf(round(degrees(phi + PI/2)),3);  
  String S = nf(round(degrees(PI/2 - A - tau + PI/2)), 3);
  String E = nf(round(degrees(PI - C + PI/6)),3);
  String W = nf(round(degrees(PI - m + PI/2)),3);
  String Wrist2_s = nf(0,3); // Wrist2 remains always 0 in our simplified geometric model
  String End_effector_s = nf(End_effector,3);
      
  // Write angle values (string format) to serial port:
  myPort.write(Base_s);
  delay(10);  
  myPort.write(S);
  delay(10);
  myPort.write(E);
  delay(10);
  myPort.write(W);
  delay(10);
  myPort.write(Wrist2_s);
  delay(10);
  myPort.write(End_effector_s);
  delay(10);
  myPort.write("\r\n");  // Write empty line to serial port  

  println(Base_s, S, E, W, Wrist2_s, End_effector_s);  // Servo angles can be printed in Processing terminal for debugging 
}